@@ -1,4 +1,6 @@
 
+# $rcs = ' $Id: Charset.pm,v 1.35 2004/01/15 12:00:04 Daddy Exp $ ' ;
+
 =head1 NAME
 
 I18N::Charset - IANA Character Set Registry names and Unicode::MapUTF8
@@ -18,11 +20,14 @@ I18N::Charset - IANA Character Set Registry names and Unicode::MapUTF8
   # $sCharset is now 'sjis' which can be passed to Unicode::MapUTF8->new()
   $sCharset = libi_charset_name('x-sjis');
   # $sCharset is now 'MS_KANJI' which can be passed to `iconv -f $sCharset ...`
+  $sCharset = enco_charset_name('Shift-JIS');
+  # $sCharset is now 'shiftjis' which can be passed to `Encode::from_to()`
 
   I18N::Charset::add_iana_alias('my-japanese' => 'iso-2022-jp');
   I18N::Charset::add_map8_alias('my-arabic' => 'arabic7');
   I18N::Charset::add_umap_alias('my-hebrew' => 'ISO-8859-8');
   I18N::Charset::add_libi_alias('my-sjis' => 'x-sjis');
+  I18N::Charset::add_enco_alias('my-japanese' => 'shiftjis');
 
 =cut
 
@@ -45,16 +50,22 @@ So, for example, if you get an HTML document with a META CHARSET="..."
 tag, you can fairly quickly determine what Unicode::MapXXX module can
 be used to convert it to Unicode.
 
+If you don't have the module Unicode::Map installed, the umap_
+functions will always return undef.
 If you don't have the module Unicode::Map8 installed, the map8_
-functions will always return undef; similarly for Unicode::Map and the
-umap_ functions.
+functions will always return undef.
+If you don't have the module Unicode::MapUTF8 installed, the umu8_
+functions will always return undef.
+If you don't have the iconv library installed, the libi_
+functions will always return undef.
+If you don't have the Encode module installed, the enco_
+functions will always return undef.
 
 =cut
 
 #-----------------------------------------------------------------------
 
 require Exporter;
-use File::Which;
 use Carp;
 use IO::String;
 
@@ -65,7 +76,7 @@ use strict;
 #	Public Global Variables
 #-----------------------------------------------------------------------
 use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK );
-$VERSION = '1.23';
+$VERSION = sprintf("%d.%02d", q$Revision: 1.35 $ =~ /(\d+)\.(\d+)/o);
 @ISA       = qw( Exporter );
 @EXPORT    = qw( iana_charset_name
 map8_charset_name
@@ -74,9 +85,10 @@ umu8_charset_name
 mib_charset_name
 mime_charset_name
 libi_charset_name
+enco_charset_name
 mib_to_charset_name charset_name_to_mib
  );
-@EXPORT_OK = qw( add_iana_alias add_map8_alias add_umap_alias add_libi_alias );
+@EXPORT_OK = qw( add_iana_alias add_map8_alias add_umap_alias add_libi_alias add_enco_alias );
 
 #-----------------------------------------------------------------------
 #	Private Global Variables
@@ -104,10 +116,11 @@ my %MIBtoUMU8;
 # %MIBtoLIBI is a hash of mib to libiconv names.  (Only valid for
 # those libiconv names that we can find in the IANA registry)
 my %MIBtoLIBI;
+# %MIBtoLIBI is a hash of mib to Encode names.  (Only valid for
+# those Encode names that we can find in the IANA registry)
+my %MIBtoENCO;
 
-my ($debug, $verbose);
-# $debug = 1;
-$verbose ||= $debug;
+use constant DEBUG => 0;
 
 =head1 CONVERSION ROUTINES
 
@@ -129,19 +142,28 @@ within the string are not important.
 
 =cut
 
+my $sDummy = 'dummymib';
+my $sFakeMIB = $sDummy .'001';
+
+sub _is_dummy
+  {
+  my $s = shift;
+  return ($s =~ m!\A$sDummy!);
+  } # _is_dummy
+
 sub iana_charset_name
   {
   my $code = shift;
   return undef unless defined $code;
   return undef unless $code ne '';
-  # $debug = ($code =~ m!sjis!);
-  # print STDERR " + iana_charset_name($code)..." if $debug;
+  # $iDebug = ($code =~ m!sjis!);
+  # print STDERR " + iana_charset_name($code)..." if $iDebug;
   my $mib = &short_to_mib($code);
   return undef unless defined $mib;
-  # print STDERR " + mib is ($mib)..." if $debug;
+  # print STDERR " + mib is ($mib)..." if $iDebug;
   # Make sure this is really a IANA mib:
-  return undef if ($mib =~ m!\Adummymib!);
-  # print STDERR " + is really iana..." if $debug;
+  return undef if &_is_dummy($mib);
+  # print STDERR " + is really iana..." if $iDebug;
   return $MIBtoLONG{$mib};
   } # iana_charset_name
 
@@ -158,20 +180,20 @@ sub short_to_mib
   {
   my $code = shift;
   local $^W = 0;
-  # print STDERR " + short_to_mib($code)..." if $debug;
+  # print STDERR " + short_to_mib($code)..." if DEBUG;
   my $answer = undef;
  TRY_SHORT:
   foreach my $sTry (&try_list($code))
     {
     my $iMIB = $SHORTtoMIB{$sTry} || 'undef';
-    # print STDERR "try($sTry)...$iMIB..." if $debug;
+    # print STDERR "try($sTry)...$iMIB..." if DEBUG;
     if ($iMIB ne 'undef')
       {
       $answer = $iMIB;
       last TRY_SHORT;
       } # if
     } # foreach
-  # print STDERR "answer is $answer\n" if $debug;
+  # print STDERR "answer is $answer\n" if DEBUG;
   return $answer;
   } # short_to_mib
 
@@ -180,7 +202,7 @@ sub short_to_long
   {
   local $^W = 0;
   my $s = shift;
-  print STDERR " + short_to_long($s)..." if $debug;
+  # print STDERR " + short_to_long($s)..." if DEBUG;
   return $MIBtoLONG{&short_to_mib($s)};
   } # short_to_long
 
@@ -203,17 +225,91 @@ sub mime_charset_name
   my $code = shift;
   return undef unless defined $code;
   return undef unless $code ne '';
-  # print STDERR " + mime_charset_name($code)..." if $debug;
+  # print STDERR " + mime_charset_name($code)..." if DEBUG;
   my $mib = &short_to_mib($code);
   return undef unless defined $mib;
-  # print STDERR " + mib is ($mib)..." if $debug;
+  # print STDERR " + mib is ($mib)..." if DEBUG;
   # Make sure this is really an IANA mib:
-  return undef if ($mib =~ m!\Adummymib!);
-  # print STDERR " + is really iana..." if $debug;
+  return undef if &_is_dummy($mib);
+  # print STDERR " + is really iana..." if DEBUG;
   return $MIBtoMIME{$mib};
   } # mime_charset_name
 
 
+=item enco_charset_name()
+
+This function takes a string containing the name of a character set
+and returns a string which contains a name of the character set
+suitable to be passed to the Encode module.  If no valid character set
+name can be identified, or if Encode is not installed, then C<undef>
+will be returned.  The case and punctuation within the string are not
+important.
+
+    $sCharset = enco_charset_name('Extended_UNIX_Code_Packed_Format_for_Japanese');
+
+=cut
+
+my $iEncoLoaded = 0;
+
+sub _maybe_load_enco # PRIVATE
+  {
+  return if $iEncoLoaded;
+  # Get a list of aliases from Encode:
+  if (eval q{require Encode})
+    {
+    my @as = Encode->encodings(':all');
+ ENCODING:
+    foreach my $s (@as)
+      {
+      # First, see if this already has an IANA mapping:
+      my $mib;
+      my $sIana = &iana_charset_name($s);
+      if (!defined $sIana)
+        {
+        # Create a dummy mib:
+        $mib = $sFakeMIB++;
+        } # if
+      else
+        {
+        $mib = &charset_name_to_mib($sIana);
+        }
+      # At this point we have a mib for this Encode entry.
+      $MIBtoENCO{$mib} = $s;
+      # print STDERR " +   mib for enco ==$s== is $mib\n";
+      $SHORTtoMIB{&strip($s)} = $mib;
+      # print STDERR " +   assign enco =$s==>$mib\n" if &_is_dummy($mib);
+      } # if
+    $iEncoLoaded = 1;
+    } # if
+  else
+    {
+    print STDERR " --- Encode is not installed\n";
+    }
+  } # _maybe_load_enco
+
+sub _mib_to_enco # PRIVATE
+  {
+  &_maybe_load_enco();
+  return $MIBtoENCO{shift()};
+  } # _mib_to_enco
+
+sub enco_charset_name
+  {
+  my $code = shift;
+  return undef unless defined $code;
+  return undef unless $code ne '';
+  &_maybe_load_enco();
+  my $iDebug = 0; # ($code =~ m!johab!i);
+  print STDERR " + enco_charset_name($code)..." if $iDebug;
+  my $mib = &short_to_mib($code);
+  return undef unless defined $mib;
+  print STDERR " + mib is ($mib)..." if $iDebug;
+  my $ret = &_mib_to_enco($mib);
+  print STDERR " + enco is ($ret)..." if $iDebug;
+  return $ret;
+  } # enco_charset_name
+
+
 =item libi_charset_name()
 
 This function takes a string containing the name of a character set
@@ -226,18 +322,76 @@ within the string are not important.
 
 =cut
 
+my $iLibiLoaded = 0;
+
+sub _maybe_load_libi # PRIVATE
+  {
+  return if $iLibiLoaded;
+  # Get a list of aliases from iconv:
+  return unless eval 'require App::Info::Lib::Iconv';
+  my $oAILI = new App::Info::Lib::Iconv;
+  if (ref $oAILI)
+    {
+    my $iLibiVersion = $oAILI->version;
+    # print STDERR " + libiconv version is $iLibiVersion\n";
+    if ($oAILI->installed && (1.8 <= $iLibiVersion))
+      {
+      my $sCmd = $oAILI->bin_dir . '/iconv -l';
+      # print STDERR " + iconv cmdline is $sCmd\n";
+      my @asIconv = split(/\n/, `$sCmd`);
+ ICONV_LINE:
+      foreach my $sLine (@asIconv)
+        {
+        my @asWord = split(/\s+/, $sLine);
+        # First, go through and find one of these that has an IANA mapping:
+        my $mib;
+        my $sIana = undef;
+ FIND_IANA:
+        foreach my $sWord (@asWord)
+          {
+          last FIND_IANA if ($sIana = &iana_charset_name($sWord));
+          } # foreach FIND_IANA
+        if (!defined $sIana)
+          {
+          # Create a dummy mib:
+          $mib = $sFakeMIB++;
+          } # if
+        else
+          {
+          $mib = &charset_name_to_mib($sIana);
+          }
+        # At this point we have a mib for this iconv entry.  Assign them all:
+ ADD_LIBI:
+        foreach my $sWord (reverse @asWord)
+          {
+          $MIBtoLIBI{$mib} = $sWord;
+          # print STDERR " +   mib for libi ==$sWord== is $mib\n";
+          $SHORTtoMIB{&strip($sWord)} = $mib;
+          } # foreach ADD_LIBI
+        } # foreach ICONV_LINE
+      } # if
+    } # if
+  $iLibiLoaded = 1;
+  } # _maybe_load_libi
+
+sub _mib_to_libi # PRIVATE
+  {
+  &_maybe_load_libi();
+  return $MIBtoLIBI{shift()};
+  } # _mib_to_libi
+
 sub libi_charset_name
   {
   my $code = shift;
   return undef unless defined $code;
   return undef unless $code ne '';
-  # my $debug = 1; # ($code =~ m!johab!i);
-  # print STDERR " + libi_charset_name($code)..." if $debug;
+  # my $iDebug = 1; # ($code =~ m!johab!i);
+  # print STDERR " + libi_charset_name($code)..." if $iDebug;
   my $mib = &short_to_mib($code);
   return undef unless defined $mib;
-  # print STDERR " + mib is ($mib)..." if $debug;
-  my $ret = $MIBtoLIBI{$mib};
-  # print STDERR " + libi is ($ret)..." if $debug;
+  # print STDERR " + mib is ($mib)..." if $iDebug;
+  my $ret = &_mib_to_libi($mib);
+  # print STDERR " + libi is ($ret)..." if $iDebug;
   return $ret;
   } # libi_charset_name
 
@@ -277,7 +431,7 @@ non-IANA character sets, this is an unambiguous unique string whose
 only use is to pass to other functions in this module.  If no valid
 character set name can be identified, then C<undef> will be returned.
 
-    $iMIB = charset_name_mib('US-ASCII');
+    $iMIB = charset_name_to_mib('US-ASCII');
 
 =cut
 
@@ -312,18 +466,18 @@ sub map8_charset_name
   my $code = shift;
   return undef unless defined $code;
   return undef unless $code ne '';
-  $debug = 0 && ($code =~ m!037!);
-  print STDERR " + map8_charset_name($code)..." if $debug;
+  # $iDebug = 0 && ($code =~ m!037!);
+  # print STDERR " + map8_charset_name($code)..." if $iDebug;
   $code = &strip($code);
-  print STDERR "$code..." if $debug;
+  # print STDERR "$code..." if $iDebug;
   my $iMIB = &short_to_mib($code) || 'undef';
-  print STDERR "$iMIB..." if $debug;
+  # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
-    print STDERR "$MIBtoMAP8{$iMIB}\n" if $debug;
+    # print STDERR "$MIBtoMAP8{$iMIB}\n" if $iDebug;
     return $MIBtoMAP8{$iMIB};
     } # if
-  print STDERR "undef\n" if $debug;
+  # print STDERR "undef\n" if $iDebug;
   return undef;
   } # map8_charset_name
 
@@ -347,17 +501,17 @@ sub umap_charset_name
   my $code = shift;
   return undef unless defined $code;
   return undef unless $code ne '';
-  # $debug = ($code =~ m!apple!i);
-  # print STDERR "\n + MIBtoUMAP{dummymib029} == $MIBtoUMAP{'dummymib029'}\n\n" if $debug;
-  # print STDERR " + umap_charset_name($code)..." if $debug;
+  # $iDebug = ($code =~ m!apple!i);
+  # print STDERR "\n + MIBtoUMAP{dummymib029} == $MIBtoUMAP{$sDummy .'029'}\n\n" if $iDebug;
+  # print STDERR " + umap_charset_name($code)..." if $iDebug;
   my $iMIB = &short_to_mib(&strip($code)) || 'undef';
-  # print STDERR "$iMIB..." if $debug;
+  # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
-    # print STDERR "$MIBtoUMAP{$iMIB}\n" if $debug;
+    # print STDERR "$MIBtoUMAP{$iMIB}\n" if $iDebug;
     return $MIBtoUMAP{$iMIB};
     } # if
-  # print STDERR "undef\n" if $debug;
+  # print STDERR "undef\n" if $iDebug;
   return undef;
   } # umap_charset_name
 
@@ -369,7 +523,7 @@ This function takes a string containing the name of a character set
 the character set that can be passed to Unicode::MapUTF8::new(). If no
 valid character set name can be identified, then C<undef> will be
 returned.  The case and punctuation within the argument string are not
-important.  
+important.
 
     $sCharset = umu8_charset_name('windows-1251');
 
@@ -380,16 +534,16 @@ sub umu8_charset_name
   my $code = shift;
   return undef unless defined $code;
   return undef unless $code ne '';
-  # $debug = ($code =~ m!u!);
-  # print STDERR " + umu8_charset_name($code)..." if $debug;
+  # $iDebug = ($code =~ m!u!);
+  # print STDERR " + umu8_charset_name($code)..." if $iDebug;
   my $iMIB = &short_to_mib($code) || 'undef';
-  # print STDERR "$iMIB..." if $debug;
+  # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
-    # print STDERR "$MIBtoUMU8{$iMIB}\n" if $debug;
+    # print STDERR "$MIBtoUMU8{$iMIB}\n" if $iDebug;
     return $MIBtoUMU8{$iMIB};
     } # if
-  # print STDERR "undef\n" if $debug;
+  # print STDERR "undef\n" if $iDebug;
   return undef;
   } # umu8_charset_name
 
@@ -448,20 +602,19 @@ name referred to by the existing alias 'sjis' (which happens to be 'Shift_JIS').
 
 =cut
 
-#-----------------------------------------------------------------------
-
 sub add_iana_alias
   {
   my ($sAlias, $sReal) = @_;
-  # print STDERR " + add_iana_alias($sAlias,$sReal)...";
+  # print STDERR " + add_iana_alias($sAlias, $sReal)\n";
   my $sName = &iana_charset_name($sReal);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown IANA character set \"$sReal\"\n" if $verbose;
+    carp "attempt to alias \"$sAlias\" to unknown IANA character set \"$sReal\"\n" if DEBUG;
     return undef;
     } # if
-  # print STDERR "$sName...", &short_to_mib(&strip($sName)), "\n";
-  $SHORTtoMIB{&strip($sAlias)} = &short_to_mib(&strip($sName));
+  my $mib = &short_to_mib(&strip($sName));
+  # print STDERR " --> $sName --> $mib\n";
+  $SHORTtoMIB{&strip($sAlias)} = $mib;
   return $sName;
   } # add_iana_alias
 
@@ -503,8 +656,6 @@ aliases for 'EBCDIC-CA-FR'.
 
 =cut
 
-#-----------------------------------------------------------------------
-
 sub add_map8_alias
   {
   my ($sAlias, $sReal) = @_;
@@ -513,7 +664,7 @@ sub add_map8_alias
   my $sShortName = &strip($sName);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown Map8 character set \"$sReal\"\n" if $verbose;
+    carp "attempt to alias \"$sAlias\" to unknown Map8 character set \"$sReal\"\n" if DEBUG;
     return undef;
     } # if
   if (exists $SHORTtoMIB{$sShortName})
@@ -540,7 +691,7 @@ sub add_umap_alias
   my $sShortName = &strip($sName);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown U::Map character set \"$sReal\"\n" if $verbose;
+    carp "attempt to alias \"$sAlias\" to unknown U::Map character set \"$sReal\"\n" if DEBUG;
     return undef;
     } # if
   if (exists $SHORTtoMIB{$sShortName})
@@ -550,6 +701,8 @@ sub add_umap_alias
   return $sName;
   } # add_umap_alias
 
+#-----------------------------------------------------------------------
+
 =item  add_libi_alias()
 
 This function takes two strings: a new alias, and a target iconv
@@ -562,21 +715,19 @@ Returns 'undef' if there is no such target conversion scheme or alias.
 
 Examples:
 
-  I18N::Charset::add_libi_alias('my-chinese' => 'CN-GB');
+  I18N::Charset::add_libi_alias('my-chinese1' => 'CN-GB');
 
-With this code, "my-chinese" becomes an alias for the existing iconv
+With this code, "my-chinese1" becomes an alias for the existing iconv
 conversion scheme 'CN-GB'.
 
   I18N::Charset::add_libi_alias('my-chinese2' => 'EUC-CN');
 
-With this code, "my-chinese2" becomes an alias for the LIBI conversion
-scheme referred to by the existing alias 'EUC-CN' (which happens to be
-'CN-GB').
+With this code, "my-chinese2" becomes an alias for the iconv
+conversion scheme referred to by the existing alias 'EUC-CN' (which
+happens to be 'CN-GB').
 
 =cut
 
-#-----------------------------------------------------------------------
-
 sub add_libi_alias
   {
   my ($sAlias, $sReal) = @_;
@@ -584,7 +735,7 @@ sub add_libi_alias
   my $sName = &libi_charset_name($sReal);
   if (not defined($sName))
     {
-    carp "attempt to alias \"$sAlias\" to unknown iconv conversion scheme \"$sReal\"\n" if $verbose;
+    carp "attempt to alias \"$sAlias\" to unknown iconv conversion scheme \"$sReal\"\n" if DEBUG;
     return undef;
     } # if
   my $mib = &short_to_mib(&strip($sName));
@@ -595,6 +746,50 @@ sub add_libi_alias
 
 #-----------------------------------------------------------------------
 
+=item  add_enco_alias()
+
+This function takes two strings: a new alias, and a target Encode
+encoding Name (or existing Encode alias).  It defines the new alias
+referring to that encoding name (or to the encoding to which the
+existing alias refers).
+
+Returns the target encoding name of the successfully installed alias.
+Returns 'undef' if there is no such encoding or alias.
+
+Examples:
+
+  I18N::Charset::add_enco_alias('my-japanese1' => 'jis0201-raw');
+
+With this code, "my-japanese1" becomes an alias for the existing
+encoding 'jis0201-raw'.
+
+  I18N::Charset::add_enco_alias('my-japanese2' => 'my-japanese1');
+
+With this code, "my-japanese2" becomes an alias for the encoding
+referred to by the existing alias 'my-japanese1' (which happens to be
+'jis0201-raw' after the previous call).
+
+=cut
+
+sub add_enco_alias
+  {
+  my ($sAlias, $sReal) = @_;
+  my $iDebug = 0;
+  print STDERR " + add_enco_alias($sAlias,$sReal)..." if $iDebug;
+  my $sName = &enco_charset_name($sReal);
+  if (not defined($sName))
+    {
+    carp "attempt to alias \"$sAlias\" to unknown iconv conversion scheme \"$sReal\"\n" if DEBUG;
+    return undef;
+    } # if
+  my $mib = &short_to_mib(&strip($sName));
+  print STDERR "sName=$sName...mib=$mib\n" if $iDebug;
+  $SHORTtoMIB{&strip($sAlias)} = $mib;
+  return $sName;
+  } # add_enco_alias
+
+#-----------------------------------------------------------------------
+
 =head1 EXAMPLES
 
 =head1 KNOWN BUGS AND LIMITATIONS
@@ -680,6 +875,7 @@ INFINITE:
 while (1)
   {
   my ($sName, $iMIB, $sAlias, $mimename);
+  my $iDebug = 0;
   # I used to use the __DATA__ mechanism to initialize the data, but
   # that is not compatible with perlapp
   my $io = new IO::String(_init_data());
@@ -691,10 +887,9 @@ while (1)
       {
       # Blank line separates entries.
       ($sName, $iMIB, $sAlias, $mimename) = ('', '', '', 0);
-      $debug = 0;
       next DATA;
       } # unless
-    # print STDERR " + read DATA $_" if $debug;
+    # print STDERR " + read DATA $_" if $iDebug;
 
     # If this line indicates the MIME name, remember this fact; but if
     # it doesn't, keep the previous setting (in case the MIME name is
@@ -703,20 +898,20 @@ while (1)
     if ($sLine =~ m/^Name:\s*(\S+)/)
       {
       $sName = $1;
-      # $debug = ($sName =~ m!1252!i);
-      print STDERR " +   read Name: $sName\n" if $debug;
+      # $iDebug = ($sName =~ m!1252!i);
+      print STDERR " +   read Name: $sName\n" if $iDebug;
       } # if Name
     elsif ($sLine =~ m!^MIBenum:\s*(\d+)!)
       {
       $iMIB = $1;
-      # $debug = ($iMIB =~ m!225[23]!);
-      print STDERR " +   found mib: $iMIB, long = $sName\n" if $debug;
+      # $iDebug = ($iMIB =~ m!225[23]!);
+      print STDERR " +   found mib: $iMIB, long = $sName\n" if $iDebug;
       $MIBtoLONG{$iMIB} = $sName;
       $LONGtoMIB{$sName} = $iMIB;
       $SHORTtoMIB{&strip($sName)} = $iMIB;
       if ($mimename)
         {
-        print STDERR " +   found mime: mib=$iMIB, mime=$sName\n" if $debug;
+        print STDERR " +   found mime: mib=$iMIB, mime=$sName\n" if $iDebug;
 	$MIBtoMIME{$iMIB} = $sName;
         # We've got the MIME name for this entry, don't look for more
         # until next entry:
@@ -728,11 +923,11 @@ while (1)
       $sAlias = $1;
       if ($sAlias !~ m!None!i)
         {
-        print STDERR " +   map Alias ($sAlias) to mib $iMIB\n" if $debug;
+        print STDERR " +   map Alias ($sAlias) to mib $iMIB\n" if $iDebug;
         $SHORTtoMIB{&strip($sAlias)} = $iMIB;
 	if ($mimename)
 	  {
-          print STDERR " +   found mime: mib=$iMIB, mime=$sAlias\n" if $debug;
+          print STDERR " +   found mime: mib=$iMIB, mime=$sAlias\n" if $iDebug;
 	  $MIBtoMIME{$iMIB} = $sAlias;
           # We've got the MIME name for this entry, don't look for
           # more until next entry:
@@ -773,18 +968,16 @@ while (1)
 
   # last INFINITE;
 
-  my $sFakeMIB = 'dummymib001';
-
   if (eval "require Unicode::Map8")
     {
-    # $debug = 1;
-    print STDERR " + found Unicode::Map8 installed, will build map8 tables..." if $debug;
+    # $iDebug = 1;
+    print STDERR " + found Unicode::Map8 installed, will build map8 tables..." if $iDebug;
     my $sMapFile = "$Unicode::Map8::MAPS_DIR/aliases";
     if (open MAPS, $sMapFile)
       {
       while (defined (my $sLine = <MAPS>))
         {
-        # $debug = ($sLine =~ m!866!);
+        # $iDebug = ($sLine =~ m!866!);
         my @as = split(/\s/, $sLine);
         my $sFound = '';
         my $iMIB = '';
@@ -794,7 +987,7 @@ while (1)
  MAP8ITEM:
         foreach my $s ($sMap8, @as)
           {
-          print STDERR " +      looking for $s in iana table...\n" if $debug;
+          print STDERR " +      looking for $s in iana table...\n" if $iDebug;
           if (defined (my $sTemp = &iana_charset_name($s)))
             {
             $sFound = $sTemp;
@@ -803,46 +996,46 @@ while (1)
           } # foreach
         if ($sFound eq '')
           {
-          # $debug = 1;
+          # $iDebug = 1;
           $iMIB = $sFakeMIB++;
-          print STDERR " +   had to use a dummy mib ($iMIB) for U::Map8==$sMap8==\n" if $debug;
+          print STDERR " +   had to use a dummy mib ($iMIB) for U::Map8==$sMap8==\n" if $iDebug;
           $LONGtoMIB{$sMap8} = $iMIB;
           } # unless
         else
           {
           $iMIB = $LONGtoMIB{$sFound};
-          print STDERR " +   found IANA name $sFound ($iMIB) for Map8 entry $sMap8\n" if $debug;
+          print STDERR " +   found IANA name $sFound ($iMIB) for Map8 entry $sMap8\n" if $iDebug;
           }
-        # $debug = ($iMIB =~ m!225[23]!);
+        # $iDebug = ($iMIB =~ m!225[23]!);
         # Make this IANA mib map to this Map8 name:
-        print STDERR " +      map $iMIB to $sMap8 in MIBtoMAP8...\n" if $debug;
+        print STDERR " +      map $iMIB to $sMap8 in MIBtoMAP8...\n" if $iDebug;
         $MIBtoMAP8{$iMIB} = $sMap8;
         # Map ALL the Map8 aliases to this Map8 master name:
         foreach my $s ($sMap8, @as)
           {
           $s = &strip($s);
-          print STDERR " +      map $s to $iMIB in SHORTtoMIB...\n" if $debug;
+          print STDERR " +      map $s to $iMIB in SHORTtoMIB...\n" if $iDebug;
           $SHORTtoMIB{$s} = $iMIB;
           } # foreach
-        # $debug = 0;
+        # $iDebug = 0;
         } # while
       close MAPS;
       } # if open
     else
       {
-      carp " --- couldn't open $sMapFile for read" if $verbose;
+      carp " --- couldn't open $sMapFile for read" if $iDebug;
       }
     # If there are special cases for Unicode::Map8, add them here:
     # &add_map8_alias("new-name", "existing-name");
-    print STDERR "done.\n" if $debug;
+    print STDERR "done.\n" if $iDebug;
     } # if Unicode::Map8 installed
 
   # last INFINITE;
 
-  # $debug = 1;
+  # $iDebug = 1;
   if (eval "require Unicode::Map")
     {
-    print STDERR " + found Unicode::Map installed, will build tables..." if $debug;
+    print STDERR " + found Unicode::Map installed, will build tables..." if $iDebug;
     my $MAP_Path = $INC{'Unicode/Map.pm'};
     $MAP_Path =~ s/\.pm//;
     my $sMapFile = "$MAP_Path/REGISTRY";
@@ -853,14 +1046,14 @@ while (1)
       my @asMAPS = split(/\n\s*\n/, <MAPS>);
       foreach my $sEntry (@asMAPS)
         {
-        $debug = 0;
+        $iDebug = 0;
         # print STDERR " + working on Umap entry >>>>>$sEntry<<<<<...\n";
         my ($sName, $iMIB) = ('', '');
         # Get the value of the name field, and skip entries with no name:
         next unless $sEntry =~ m!^name:\s+(\S+)!mi;
         $sName = $1;
-        # $debug = ($sName =~ m!apple!);
-        print STDERR " +   UMAP sName is $sName\n" if $debug;
+        # $iDebug = ($sName =~ m!apple!);
+        print STDERR " +   UMAP sName is $sName\n" if $iDebug;
         my @asAlias = split /\n/, $sEntry;
         @asAlias = map { /alias:\s+(.*)/; $1 } (grep /alias/, @asAlias);
         # See if this entry already has the MIB identified:
@@ -875,11 +1068,11 @@ while (1)
  ALIAS:
           foreach my $sAlias ($sName, @asAlias)
             {
-            print STDERR " +     try alias $sAlias\n" if $debug;
+            print STDERR " +     try alias $sAlias\n" if $iDebug;
             my $iMIBtry = &short_to_mib(&strip($sAlias));
             if ($iMIBtry)
               {
-              print STDERR " +       matched\n" if $debug;
+              print STDERR " +       matched\n" if $iDebug;
               $iMIB = $iMIBtry;
               last ALIAS;
               } # if
@@ -888,12 +1081,12 @@ while (1)
           if ($iMIB eq '')
             {
             $iMIB = $sFakeMIB++;
-            print STDERR " +   had to use a dummy mib ($iMIB) for U::Map==$sName==\n" if $debug;
+            print STDERR " +   had to use a dummy mib ($iMIB) for U::Map==$sName==\n" if $iDebug;
             } # if
           } # else
-        # $debug = ($iMIB =~ m!225[23]!);
-        # $debug = ($iMIB eq '17');
-        print STDERR " +   UMAP mib is $iMIB\n" if $debug;
+        # $iDebug = ($iMIB =~ m!225[23]!);
+        # $iDebug = ($iMIB eq '17');
+        print STDERR " +   UMAP mib is $iMIB\n" if $iDebug;
         $MIBtoUMAP{$iMIB} = $sName;
         # Set the long IANA name, IF it is empty so far:
         $MIBtoLONG{$iMIB} ||= $sName;
@@ -901,103 +1094,66 @@ while (1)
         $SHORTtoMIB{&strip($sName)} ||= $iMIB;
         foreach my $sAlias (@asAlias)
           {
-          print STDERR " +   UMAP alias $sAlias\n" if $debug;
+          print STDERR " +   UMAP alias $sAlias\n" if $iDebug;
           $SHORTtoMIB{&strip($sAlias)} = $iMIB;
           } # while
         } # foreach
       close MAPS;
-      # print STDERR "\n + MIBtoUMAP{dummymib029} == $MIBtoUMAP{'dummymib029'}\n\n";
+      # print STDERR "\n + MIBtoUMAP{dummymib029} == $MIBtoUMAP{$sDummy .'029'}\n\n";
       } # if open
     else
       {
-      carp " --- couldn't open $sMapFile for read" if $verbose;
+      carp " --- couldn't open $sMapFile for read" if $iDebug;
       }
     # If there are special cases for Unicode::Map, add them here:
     # &add_umap_alias("new-name", "existing-name");
-    print STDERR "done.\n" if $debug;
+    print STDERR "done.\n" if $iDebug;
     } # if Unicode::Map installed
 
   # Make sure to do U::MapUTF8 last, because it (in turn) depends on
   # the others.
-  # $debug = 1;
+  # $iDebug = 1;
   if (1.0 <= (eval q{ require Unicode::MapUTF8; $Unicode::MapUTF8::VERSION } || 0))
     {
-    print STDERR " + found Unicode::MapUTF8 $Unicode::MapUTF8::VERSION installed, will build tables...\n" if $debug;
+    print STDERR " + found Unicode::MapUTF8 $Unicode::MapUTF8::VERSION installed, will build tables...\n" if $iDebug;
     my @as;
     # Wrap this in an eval to avoid compiler warning(?):
-    eval q{ @as = &Unicode::MapUTF8::utf8_supported_charset };
+    eval { @as = &Unicode::MapUTF8::utf8_supported_charset() };
 UMU8_NAME:
     foreach my $sName (@as)
       {
-      # $debug = ($sName =~ m!jis!i);
-      print STDERR " + working on UmapUTF8 entry >>>>>$sName<<<<<...\n" if $debug;
+      # $iDebug = ($sName =~ m!jis!i);
+      print STDERR " + working on UmapUTF8 entry >>>>>$sName<<<<<...\n" if $iDebug;
       my $s = iana_charset_name($sName) || '';
       if ($s ne '')
         {
-        # print STDERR " +   iana name is >>>>>$s<<<<<...\n" if $debug;
+        # print STDERR " +   iana name is >>>>>$s<<<<<...\n" if $iDebug;
         $MIBtoUMU8{charset_name_to_mib($s)} = $sName;
         next UMU8_NAME;
         } # if already maps to IANA
-      # print STDERR " +   UmapUTF8 entry ===$sName=== has no iana entry\n" if $debug;
+      # print STDERR " +   UmapUTF8 entry ===$sName=== has no iana entry\n" if $iDebug;
       $s = umap_charset_name($sName) || '';
       if ($s ne '')
         {
-        print STDERR " +   U::Map name is >>>>>$s<<<<<...\n" if $debug;
+        print STDERR " +   U::Map name is >>>>>$s<<<<<...\n" if $iDebug;
         $MIBtoUMU8{charset_name_to_mib($s)} = $sName;
         next UMU8_NAME;
         } # if maps to U::Map
-      # print STDERR " +   UmapUTF8 entry ===$sName=== has no U::Map entry\n" if $debug;
+      # print STDERR " +   UmapUTF8 entry ===$sName=== has no U::Map entry\n" if $iDebug;
       $s = map8_charset_name($sName) || '';
       if ($s ne '')
         {
-        print STDERR " +   U::Map8 name is >>>>>$s<<<<<...\n" if $debug;
+        print STDERR " +   U::Map8 name is >>>>>$s<<<<<...\n" if $iDebug;
         $MIBtoUMU8{charset_name_to_mib($s)} = $sName;
         next UMU8_NAME;
         } # if maps to U::Map8
-      print STDERR " +   UmapUTF8 entry ===$sName=== has no entries at all\n" if $debug;
+      print STDERR " +   UmapUTF8 entry ===$sName=== has no entries at all\n" if $iDebug;
       } # foreach
     # If there are special cases for Unicode::MapUTF8, add them here:
     # &add_umap_alias("new-name", "existing-name");
-    print STDERR "done.\n" if $debug;
+    print STDERR "done.\n" if $iDebug;
     } # if Unicode::MapUTF8 installed
 
-  # Get a list of aliases from iconv:
-  my $sIconv = File::Which::which('iconv') || '';
-  if ($sIconv ne '')
-    {
-    my @asIconv = split(/\n/, `$sIconv -l`);
- ICONV_LINE:
-    foreach my $sLine (@asIconv)
-      {
-      my @asWord = split(/\s+/, $sLine);
-      # First, go through and find one of these that has an IANA mapping:
-      my $mib;
-      my $sIana = undef;
- FIND_IANA:
-      foreach my $sWord (@asWord)
-        {
-        last FIND_IANA if ($sIana = &iana_charset_name($sWord));
-        } # foreach FIND_IANA
-      if (!defined $sIana)
-        {
-        # Create a dummy mib:
-        $mib = $sFakeMIB++;
-        } # if
-      else
-        {
-        $mib = &charset_name_to_mib($sIana);
-        }
-      # At this point we have a mib for this iconv entry.  Assign them all:
- ADD_LIBI:
-      foreach my $sWord (reverse @asWord)
-        {
-        $MIBtoLIBI{$mib} = $sWord;
-        $SHORTtoMIB{&strip($sWord)} = $mib;
-        # print STDERR " +   assign iconv =$sWord==>$mib\n" if ($mib =~ m!dummy!);
-        } # foreach ADD_LIBI
-      } # foreach ICONV_LINE
-    } # if
-
   last INFINITE;
 
   # Below here is debugging code:
@@ -1062,15 +1218,15 @@ sub _init_data_extra
   return <<'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
 
 Shift_JIS === sjis
-windows-1250 === winlatin2
-windows-1251 === wincyrillic
-windows-1252 === winlatin1
-windows-1253 === wingreek
-windows-1254 === winturkish
-windows-1255 === winhebrew
-windows-1256 === winarabic
-windows-1257 === winbaltic
-windows-1258 === winvietnamese
+windows-1250 === winlatin2 === cp1250
+windows-1251 === wincyrillic === cp1251
+windows-1252 === winlatin1 === cp1252
+windows-1253 === wingreek === cp1253
+windows-1254 === winturkish === cp1254
+windows-1255 === winhebrew === cp1255
+windows-1256 === winarabic === cp1256
+windows-1257 === winbaltic === cp1257
+windows-1258 === winvietnamese === cp1258
 Adobe-Standard-Encoding === adobe-standard
 Adobe-Symbol-Encoding === adobe-symbol
 EBCDIC-ES === ebcdic-cp-es
@@ -1087,6 +1243,14 @@ ISO-10646-UCS-2 === ucs2
 ISO-10646-UCS-4 === ucs4
 # These were added for iconv:
 ISO-2022-JP === ISO-2022-JP-1
+# These are for Encode:
+IBM1047 === cp1047
+GB2312 === gb2312-raw
+HZ-GB-2312 === hz
+JIS_X0201 === jis0201-raw
+JIS_C6226-1983 === jis0208-raw
+JIS_X0212-1990 === jis0212-raw
+KS_C_5601-1987 === ksc5601-raw
 XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
   } # _init_data_extra
 
@@ -1099,7 +1263,7 @@ sub _init_data
 ===================================================================
 CHARACTER SETS
 
-(last updated 2002-09-30)
+(last updated 2003-09-19)
 
 These are the official names for character sets that may be used in
 the Internet and may be referred to in Internet documentation.  These
@@ -2454,9 +2618,9 @@ MIBenum: 103
 Source: RFC 1642
 Alias: csUnicode11UTF7
 
-Name: UTF-8                                                    [RFC2279]
+Name: UTF-8                                                    [RFC-draft-yergeau-rfc2279bis-05.txt]
 MIBenum: 106
-Source: RFC 2279
+Source: RFC draft-yergeau-rfc2279bis-05.txt
 Alias: None 
 
 Name: ISO-8859-13
@@ -2921,10 +3085,9 @@ PEOPLE
 
 
 
-
-
 EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
   } # _init_data
 
 1;
 
+__END__
